use sea_orm::{ActiveModelTrait, ColumnTrait, Condition, DatabaseConnection, EntityTrait, IntoActiveModel, ModelTrait, PaginatorTrait, QueryFilter, QueryOrder};
use sea_orm::ActiveValue::Set;

use crate::{APPLICATION_CONTEXT, form, view};
use crate::common::err::{AppError, log_error};
use crate::common::Result;
use crate::entity::{article, category};
use crate::param::{CategoryParams, DelParams};

pub struct CategoryService;

impl CategoryService {
    pub async fn index(handler_name: &str, params: CategoryParams) -> Result<view::CategoryTemplate> {
        let conn = APPLICATION_CONTEXT.get::<DatabaseConnection>();

        let page = params.page(); // 当前页码
        let page_size = params.page_size(); // 每页条数，默认15
        let mut select = category::Entity::find()
            .filter(
                Condition::all()
                    .add_option(
                        params.keyword_opt().map(|n| category::Column::Name.contains(&n))
                    )
                    .add_option(params.is_del_opt().map(|n| category::Column::IsDel.eq(n)))
            );

        if let Some(ord) = params.order() {
            select = select.order_by(category::Column::Id, ord);
        }

        let paginator = select.paginate(conn, page_size);

        let page_total = paginator.num_pages().await.map_err(AppError::from)?;

        if page_total == 0 {
            return Ok(view::CategoryTemplate {
                categories: vec![],
                params,
                page_total,
            });
        }

        let categories = paginator
            .fetch_page(page)
            .await
            .map_err(AppError::from)
            .map_err(log_error(handler_name))?;

        Ok(view::CategoryTemplate {
            categories,
            params,
            page_total,
        })
    }

    pub async fn find(handler_name: &str, id: u32) -> Result<category::Model> {
        let conn = APPLICATION_CONTEXT.get::<DatabaseConnection>();

        let cate = category::Entity::find_by_id(id)
            .one(conn)
            .await
            .map_err(AppError::from)
            .map_err(log_error(handler_name))?;

        match cate {
            None => Err(AppError::notfound()),
            Some(cate) => Ok(cate),
        }
    }

    pub async fn add(handler_name: &str, frm: form::CategoryForm) -> Result<u32> {
        let conn = APPLICATION_CONTEXT.get::<DatabaseConnection>();

        let am = category::ActiveModel {
            name: Set(frm.name),
            // all other attributes are `NotSet`
            ..Default::default()
        };

        let added_category = am.insert(conn)
            .await
            .map_err(AppError::from)
            .map_err(log_error(handler_name))?;

        Ok(added_category.id)
    }

    pub async fn edit_ui(handler_name: &str, id: u32) -> Result<view::CategoryEditTemplate> {
        let conn = APPLICATION_CONTEXT.get::<DatabaseConnection>();

        let cate = category::Entity::find_by_id(id)
            .one(conn)
            .await
            .map_err(AppError::from)
            .map_err(log_error(handler_name))?
            .ok_or(AppError::notfound())?;

        Ok(view::CategoryEditTemplate { category: cate })
    }

    pub async fn edit(handler_name: &str, id: u32, frm: form::CategoryForm) -> Result<()> {
        let conn = APPLICATION_CONTEXT.get::<DatabaseConnection>();

        let cate = category::Entity::find_by_id(id)
            .one(conn)
            .await
            .map_err(AppError::from)
            .map_err(log_error(handler_name))?
            .ok_or(AppError::notfound())?;

        let mut am = cate.into_active_model();
        am.name = Set(frm.name);

        am.save(conn)
            .await
            .map_err(AppError::from)
            .map_err(log_error(handler_name))?;

        // am.update(conn)
        //     .await
        //     .map_err(AppError::from)
        //     .map_err(log_error(handler_name))?;

        Ok(())
    }


    pub async fn del(handler_name: &str, params: DelParams) -> Result<()> {
        let conn = APPLICATION_CONTEXT.get::<DatabaseConnection>();
        let real = params.real.unwrap_or(false);

        let cate = category::Entity::find_by_id(params.id)
            .one(conn)
            .await
            .map_err(AppError::from)
            .map_err(log_error(handler_name))?
            .ok_or(AppError::notfound())?;

        if real {
            cate.delete(conn)
                .await
                .map_err(AppError::from)
                .map_err(log_error(handler_name))?;
        } else {
            let mut am = cate.into_active_model();
            am.is_del = Set(true);
            am.save(conn)
                .await
                .map_err(AppError::from)
                .map_err(log_error(handler_name))?;
        }

        Ok(())
    }

    pub async fn articles(handler_name: &str, id: u32, params: CategoryParams) -> Result<view::CategoryArticlesTemplate> {
        let conn = APPLICATION_CONTEXT.get::<DatabaseConnection>();

        let cate = category::Entity::find_by_id(id)
            .one(conn)
            .await
            .map_err(AppError::from)
            .map_err(log_error(handler_name))?
            .ok_or(AppError::notfound())
            .map_err(log_error(handler_name))?;

        let paginator = cate.find_related(article::Entity)
            .paginate(conn, params.page_size());

        let articles = paginator.fetch_page(params.page())
            .await
            .map_err(AppError::from)
            .map_err(log_error(handler_name))?;

        let page_total = paginator
            .num_pages()
            .await
            .map_err(AppError::from)
            .map_err(log_error(handler_name))?;

        Ok(view::CategoryArticlesTemplate {
            params,
            page_total,
            category: cate,
            articles,
        })
    }
}



