use bcrypt::DEFAULT_COST;
use chrono::Local;
use sea_orm::{ActiveModelTrait, ColumnTrait, Condition, DatabaseConnection, EntityTrait, QueryFilter};
use sea_orm::ActiveValue::Set;
use validator::{Validate};

use crate::APPLICATION_CONTEXT;
use crate::common::{err::AppError, Result};
use crate::common::err::AppErrorType;
use crate::common::jwt::JwtClaims;
use crate::entity::users;
use crate::form::UserSignForm;

pub struct UserService;


impl UserService {
    pub async fn list() -> Result<Vec<users::Model>> {
        let conn = APPLICATION_CONTEXT.get::<DatabaseConnection>();
        users::Entity::find().all(conn).await.map_err(AppError::from)
    }

    pub async fn userinfo(id: u32) -> Result<users::Model> {
        let conn = APPLICATION_CONTEXT.get::<DatabaseConnection>();
        let user = users::Entity::find_by_id(id).one(conn).await.map_err(AppError::from)?;
        match user {
            None => Err(AppError::notfound()),
            Some(user) => Ok(user),
        }
    }

    pub async fn sign_in(jsn: UserSignForm) -> Result<String> {
        jsn.validate().map_err(AppError::from)?;

        let conn = APPLICATION_CONTEXT.get::<DatabaseConnection>();

        let condition = Condition::all().add(users::Column::Email.eq(&jsn.email));

        let user = users::Entity::find().filter(condition).one(conn).await?;

        if user.is_some() {
            return Err(AppError::from_msg("用户已存在", AppErrorType::Database));
        }

        let hashed_password = bcrypt::hash(&jsn.password, DEFAULT_COST).unwrap();

        let user = users::ActiveModel {
            email: Set(jsn.email),
            password_hash: Set(hashed_password),
            create_time: Set(Local::now()),
            ..Default::default()
        };

        let user = user.save(conn).await.map_err(AppError::from)?;

        let jwt_token = JwtClaims::new(user.id.unwrap(), user.email.unwrap()).sign();

        jwt_token
    }
}

